#include <clooptools.h>
#include <fstream.h>
#include "MSSMparam.h"
#include "SUSYnum.h"

#include "qag.h"
#include "mcdefs.h"	
	
#define BORN	     1
#define VIRTUAL      2
#define REAL	     4

#define SM	1
#define ALL	2
	
double csvirt(double s, double t);
double csvirtsm(double s, double t);

double born(double s);
double dborn(double s, double CT);

//double csqedisr(double s);
//double csqedfsr(double s);
double csvirtqed(double s, double t);
//double csisrfac(double s, double sqrts, double Eg);
double dcsisrfac(double s, double CT, double sqrts, double Eg);

double s, sqrts;
unsigned cls, type, numc;

RESULT_TYPE intb(Double ct)
{
  return dborn(s, ct);
}

RESULT_TYPE intf(Double t)
{
  double res; // = 0.0;
  if((cls & SM) != 0)
    res = csvirtsm(s, t);
  if((cls & ALL) != 0)
    res = csvirt(s, t);
  return res;
}

RESULT_TYPE qedv(Double t)
{
  return csvirtqed(s, t);
}


/*RESULT_TYPE collfac(Double eg)
{
  return csisrfac(s, sqrts, eg);
}*/

double egsav, cost1, cost2;
QAGCLASS dcint(FIX_RELATIVE_ERROR,0.00001);
#define CTEPS  0.001

RESULT_TYPE dcollfac(Double ct)
{
  return dcsisrfac(s, ct, sqrts, egsav);
}

RESULT_TYPE dcollfac1(Double eg)
{
  egsav = eg;
  return dcint.integrate(dcollfac, cost1, cost2);
}

RESULT_TYPE dcollfac2(Double eg)
{
  return dcsisrfac(s, cost1, sqrts, eg);
}


int main(int argc, char *argv[])
{
  double ctmin, ctmax, tmin, tmax, beta;
  mocres res, res2;
//  QAGCLASS gkint(FIX_ABSOLUTE_ERROR,0.00001);
  QAGCLASS gkint(FIX_RELATIVE_ERROR,0.00001);
	  
  if(argc >= 2)
    initppars(argv[1]);
  else
  {
    cerr << "No input file specified!" << endl;
    exit(1);
  }
  setsusypars(*argv);
  softparcorr();
  setsusypars(*argv);
  ffini();
  
  s = pars[0];
  sqrts = sqrt(s);
  ctmin = pars[98]; ctmax = pars[99];
  cls = (unsigned)(rint(pars[97]));
  type = (unsigned)(rint(pars[96]));
  numc = (unsigned)(rint(pars[95]));

  res.avgi = res.sd = 0.0;
  if((type & BORN) != 0)
  {
    if(ctmin == ctmax)
      res.avgi += TOFB * dborn(s,ctmin);
    else
      res.avgi += TOFB * gkint.integrate(intb, ctmin, ctmax);
  }
  if((type & VIRTUAL) != 0)
  {
    beta = sqrt(1-4*msmr*msmr/s);
    tmin = msmr*msmr - s/2 * (1- beta*ctmin);
    tmax = msmr*msmr - s/2 * (1- beta*ctmax);
    if(ctmin == ctmax)
      res.avgi += TOFB / (16*M_PI*s*s) * beta*s/2. * intf(tmin);
    else
      res.avgi += TOFB / (16*M_PI*s*s) * gkint.integrate(intf, tmin, tmax);
  }
  if((type & REAL) != 0)
  {
    beta = sqrt(1-4*msmr*msmr/s);
    tmin = msmr*msmr - s/2 * (1- beta*ctmin);
    tmax = msmr*msmr - s/2 * (1- beta*ctmax);
    if(ctmin == ctmax)
    {
      ctmax = ctmin + CTEPS;
      if(ctmax > 1.0)
      {
	ctmax = 1.0;  ctmin = 1.0 - CTEPS;
      }
      randomize();
      printtime();
      res2 = evalMCpoint(ISR+INTERF+FAC, s, ctmin, ctmax, numc);
      res.avgi += res2.avgi/CTEPS;
      res.sd += res2.sd/CTEPS;
      cost1 = ctmin;
      res.avgi += 2*TOFB *	// factor 2 for two collinearity
                                //  regions near imcoming e+ & e-
	      gkint.integrate(dcollfac2, PHOTENCUT, sqrt(s)/2*(1.-4*msmr*msmr/s));
      res2 = evalMCpoint(FSR, s, ctmin, ctmax, numc);
      res.avgi += res2.avgi/CTEPS;
      res.sd += res2.sd/CTEPS;
      res.avgi += TOFB * beta*s/2. * qedv(tmin);
    }
    else
    {
      randomize();
      printtime();
      if(ctmin == -1 && ctmax == 1)
        res2 = evalMCpoint(ISR+FAC, s, ctmin, ctmax, numc);
      else
        res2 = evalMCpoint(ISR+INTERF+FAC, s, ctmin, ctmax, numc);
      res.avgi += res2.avgi;
      res.sd += res2.sd;
/*      res.avgi += 2*TOFB *	// factor 2 for two collinearity
                                //  regions near imcoming e+ & e-
	      gkint.integrate(collfac, PHOTENCUT, sqrt(s)/2*(1.-4*msmr*msmr/s));
	      */
      cost1 = ctmin; cost2 = ctmax;
      res.avgi += 2*TOFB *	// factor 2 for two collinearity
                                //  regions near imcoming e+ & e-
	      gkint.integrate(dcollfac1, PHOTENCUT, sqrt(s)/2*(1.-4*msmr*msmr/s));
//      res.avgi += TOFB * csqedisr(s);
      res2 = evalMCpoint(FSR, s, ctmin, ctmax, numc);
      res.avgi += res2.avgi;
      res.sd += res2.sd;
//      res.avgi += TOFB * csqedfsr(s);
      res.avgi += TOFB * gkint.integrate(qedv, tmin, tmax);
    }
  }

  ofstream of("tint.out", ios::out );
  of << res.avgi << " " << res.sd << endl;
  of.close();

  ffexi();
  return 0;
}
